/****************************************************************************/
/*																			*/
/*							VISIN ESTEREOSCPICA							*/
/*																			*/
/****************************************************************************/


#include "4d_defin.h"

extern proyectil[];
extern tanque[];
extern estrellas[];

float Vxi = -SEP_V / 2;
float Vy = -200;
float Vz = ALTURA_PUNTO_DE_VISTA;
float Vxd = SEP_V / 2;

int proyectil_vel = PROYECTIL_VELOCIDAD;

struct movil1 stanque = { 0, 0,0,0, -45_GRADOS, 3_GRADOS };
struct movil2 sestrellas = { 180_GRADOS, 0.8_GRADOS };
struct movil3 sproyectil[NUM_PROYECTILES];


void Initialize(void);
int figura(int[],int,int,int,double);
void inicia_variables(void);
int teclado(void);
void mueve(void);
void pinta(void);
void crea_estrellas(void);
int en_campo_vision(int,int,int,int);
void pip(int);
void beep(int,int);
void cambia_pantalla(void);
void eco_teclado_off(void);
void recuadro(int,int,int,int,int);
void marco();




/****************************/
/*			MAIN			*/
/****************************/

void main(void)
{
 clrscr();
 printf("\
3D Tank Stereo Demo.                                 by Manuel Guilln Gonzlez.\n\
\n\
Using a pair of magnifying lenses at 15cm from the screen you can focus on the\n\
screen clearly, and the tank appears in 3D.\n\
\n\
KEYS ----------------------------------------------------------------------\n\
q/a             Forward/Backward\n\
o/p             Turn Left/Right\n\
SPACE           Fire (sorry, nothing to aim at yet.)\n\
");

getch();



 Initialize();

 inicia_variables();
 marco();

 while( teclado() )
 {
  mueve();
  eco_teclado_off();
  pinta();
 }

 closegraph();
}


/********************************/
/*		INICIA VARIABLES		*/
/********************************/

void inicia_variables(void)
{


int i;
crea_estrellas();

stanque.fi = -90_GRADOS;
stanque.x = 0;
stanque.y = 0;
stanque.z = 0;

for( i = 0 ; i < NUM_PROYECTILES ; i++ )
 {
  sproyectil[i].x = stanque.x;
  sproyectil[i].y = stanque.y;
  sproyectil[i].z = stanque.z + 40;
  sproyectil[i].en_aire = 0;
 }
proyectil_vel = PROYECTIL_VELOCIDAD;

}

/************************************/
/*			ECO TECLADO				*/
/*  (absorbe el eco del teclado)	*/
/************************************/

void eco_teclado_off(void)
{
 while( kbhit() )
   getch();
}


/********************************/
/*				PIP				*/
/********************************/

void pip(int cont)
{
 int i,puertobk;
 puertobk=inp(97);
 outp(97, 79);
 for( i=0 ; i < cont ; i++ );
 outp(97, puertobk);
}

/********************************************/
/*					PINTA					*/
/*  (organiza los elementos del escenario)	*/
/********************************************/
void pinta(void)
{
 int i;
 figura(estrellas, 0, 250, 0, sestrellas.fi);
 figura(tanque, stanque.x, stanque.y, stanque.z, stanque.fi-PI/2 );

 for( i = 0 ; i < NUM_PROYECTILES ; i++ )
 {
  if( sproyectil[i].en_aire )
   figura( &proyectil[0], sproyectil[i].x, sproyectil[i].y,
			sproyectil[i].z, sproyectil[i].fi-PI/2 );
 }

 cambia_pantalla();

}

/************************************************/
/*				CAMBIA PANTALLA					*/
/*   (intercambia visualpage y active page)		*/
/************************************************/

void cambia_pantalla(void)
{
 static int page = 0;



 setactivepage(!page);

 setviewport( VEN2IZQ, VENARR, VEN2DER, VENABJ, 1 );
 clearviewport();
 setviewport( VEN1IZQ, VENARR, VEN1DER, VENABJ, 1 );
 clearviewport();

 setvisualpage(!page);
 page = !page;

}


/********************************************/
/*					MUEVE					*/
/* (cambia las variables de los mviles)	*/
/********************************************/

void mueve(void)
{
 int i;
 stanque.x +=  stanque.vel * cos(stanque.fi);
 stanque.y +=  stanque.vel * sin(stanque.fi);

 sestrellas.fi += sestrellas.step_fi;

 for( i = 0 ; i < NUM_PROYECTILES ; i++ )
 {
  if( sproyectil[i].en_aire )
  {
   sproyectil[i].en_aire--;
   sproyectil[i].x += sproyectil[i].dx;
   sproyectil[i].y += sproyectil[i].dy;
  }
 }
}


/****************************************/
/*				DISPARA					*/
/****************************************/
void dispara(void)
{
 int i;
 for( i = 0 ; sproyectil[i].en_aire ; i++ ) ;
 if( i < NUM_PROYECTILES )
 {
  sproyectil[i].en_aire = TIEMPO_PROYECTIL_EN_AIRE;
  sproyectil[i].dx = proyectil_vel * cos(stanque.fi);
  sproyectil[i].dy = proyectil_vel * sin(stanque.fi);

  sproyectil[i].x = stanque.x;
  sproyectil[i].y = stanque.y;
  sproyectil[i].z = stanque.z + 40;
  sproyectil[i].fi = stanque.fi;
 }
}


/********************************************/
/*					TECLADO					*/
/********************************************/

int teclado()
{
 int c;

 if( !kbhit() ) return(1);
 c = getch();
 switch (c)
 {
  case ESC:
   return(0);
  case 'q':
   if(  stanque.vel < MAX_VEL_TANQUE )  stanque.vel += .1;
   break;
  case 'a':
   if(  stanque.vel > -MAX_VEL_TANQUE )  stanque.vel -= .1;
   break;
  case 'o':
   stanque.fi += stanque.step_fi;
   break;
  case 'p':
   stanque.fi -= stanque.step_fi;
   break;
  case ' ':
   dispara();
   break;
  case 'w':
   stanque.z++;
   break;
  case 's':
   stanque.z--;
   break;
 }
 return(1);
}


/********************************************************************/
/*						EN CAMPO DE VISIN							*/
/*  (determina si una figura se dibujar dentro de las ventanas)	*/
/********************************************************************/

int en_campo_vision( int x, int y, int z, int radio )
{
 float aux1, aux2;
 if( y <= Vy + radio + 1) return(0);

 if( (aux1=(y-Vy)) < (x+radio-Vxi) * ( aux2 = Vy / (VENANCHO/2)) )
 {
  return(0);
 }
 if( aux1 < -(x-radio-Vxi) * aux2 )
 {
  return(0);
 }
 return(1);

}


/************************************************/
/*					FIGURA						*/
/*		(traza las lneas de una figura)		*/
/************************************************/

int figura(fig, atx, aty, atz, fi)
 int fig[];		/* apunta a la cadena que define la figura */
 int atx, aty, atz;		/* coordenadas donde situar la figura */
 double fi;		/* giro sobre el eje z (rad) */

{
 float xi[MAXP], xd[MAXP], y[MAXP];	/* buffer de clculo */
 int radio;		/* radio */
 int escala;	/* escala */
 int np;		/* nmero de puntos de la figura */
 int nl;		/* nmero de lneas de la figura */
 double cosfi=cos(fi), senfi=sin(fi);
 int index = 0, indexbak;

 float x0, y0, z0, xg, yg, zg;
 int i;
 float	ygVy;


 radio = fig[index++];
 if( !en_campo_vision( atx, aty, atz, radio ) ) return(0);

 escala = fig[index++];     /* lee escala */
 np = fig[index++];		/* lee nmero de puntos */


 for( i=0 ; i < np ; i++ )
 {
  x0= escala * fig[index++];
  y0= escala * fig[index++];
  z0= escala * fig[index++];

  xg = x0 * cosfi + y0 * -senfi + atx;
  yg = x0 * senfi + y0 * cosfi + aty;
  zg = z0 + atz;

  ygVy = yg - Vy; if( ygVy == 0 ) ygVy = .0000001;

  xi[i] = ( VEN1DER - VEN1IZQ ) / 2 + Vxi + Vy * (Vxi-xg) / (ygVy);
  y[i]  = ( VENABJ - VENARR ) / 2 + Vz + Vy *  (-Vz+zg) / (ygVy);
  xd[i] = ( VEN2DER - VEN2IZQ ) / 2 + Vxd + Vy * (Vxd-xg) / (ygVy);
 }

 nl = fig[index++];        /* lee nmero de lneas */
 indexbak = index;


 /********* pinta lneas ojo izquierdo (1) ************/
 setviewport( VEN1IZQ, VENARR, VEN1DER, VENABJ, 1 );
 for( i=0 ; i<nl ; i++)
 {
  int from = fig[index++];
  int to = fig[index++];
  if( from == COLOR ) { setcolor( to ); i-- ;continue; }

  line( xi[from], y[from], xi[to], y[to] );

 }
  /* pinta lneas ojo derecho */
 setviewport( VEN2IZQ, VENARR, VEN2DER, VENABJ, 1 );
 index = indexbak;
 for( i=0 ; i<nl ; i++)
 {
  int from = fig[index++];
  int to = fig[index++];
  if( from == COLOR ) { setcolor( to ); i-- ;continue; }

  line( xd[from], y[from], xd[to], y[to] );
 }
 return(1);
}



/********************/
/*		MARCO		*/
/********************/

void marco()
{
 int flag;

 for( flag=0; flag !=2; flag++ )
 {
  setactivepage( flag );
  recuadro( VEN1IZQ-1, VENARR-1, VEN1DER+1, VENABJ+1, BLUE );
  recuadro( VEN2IZQ-1, VENARR-1, VEN2DER+1, VENABJ+1, BLUE );
 }
}



void recuadro( int x1, int y1, int x2, int y2, int color )
{
 setcolor(color);
 line(x1,y1,x2,y1);
 line(x1,y1,x1,y2);
 line(x2,y1,x2,y2);
 line(x1,y2,x2,y2);
}


/****************************************************************/
/*	INITIALIZE: Initializes the graphics system and reports 	*/
/*	any errors which occured.									*/
/****************************************************************/

int    GraphDriver;		/* The Graphics device driver			*/
int    GraphMode;		/* The Graphics mode value				*/
double AspectRatio;		/* Aspect ratio of a pixel on the screen*/
int    MaxX, MaxY;		/* The maximum resolution of the screen */
int    MaxColors;		/* The maximum # of colors available	*/
int    ErrorCode;		/* Reports any graphics errors			*/
struct palettetype palette;	/* Used to read palette info		*/


void Initialize(void)
{
  int xasp, yasp;			/* Used to read the aspect ratio*/

  GraphDriver = DETECT; 		/* Request auto-detection	*/
  initgraph( &GraphDriver, &GraphMode, "" );
  ErrorCode = graphresult();		/* Read result of initialization*/
  if( ErrorCode != grOk ){		/* Error occured during init	*/
    printf(" Graphics System Error: %s\n", grapherrormsg( ErrorCode ) );
	exit( 1 );
  }
  setgraphmode(VGAMED);

  getpalette( &palette );		/* Read the palette from board	*/
  MaxColors = getmaxcolor() + 1;	/* Read maximum number of colors*/

  MaxX = getmaxx();
  MaxY = getmaxy();			/* Read size of screen		*/

  getaspectratio( &xasp, &yasp );	/* read the hardware aspect	*/
  AspectRatio = (double)xasp / (double)yasp; /* Get correction factor	*/
  setaspectratio( xasp, yasp+3000 );


}
